@@ -24,7 +24,7 @@ DEALINGS IN THE SOFTWARE.
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Callable, ClassVar, Dict, Generator, List, Optional, Tuple, Type, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Mapping, Generator, List, Optional, Tuple, Type, TypeVar, Union
 
 from . import enums, flags, utils
 from .asset import Asset
@@ -36,13 +36,14 @@ from .permissions import PermissionOverwrite, Permissions
 from .automod import AutoModTrigger, AutoModRuleAction, AutoModPresets, AutoModRule
 from .role import Role
 from .emoji import Emoji
+from .partial_emoji import PartialEmoji
 from .member import Member
 from .scheduled_event import ScheduledEvent
 from .stage_instance import StageInstance
 from .sticker import GuildSticker
 from .threads import Thread
 from .integrations import PartialIntegration
-from .channel import StageChannel
+from .channel import ForumChannel, StageChannel, ForumTag
 
 __all__ = (
     'AuditLogDiff',
@@ -63,6 +64,8 @@ if TYPE_CHECKING:
     )
     from .types.channel import (
         PermissionOverwrite as PermissionOverwritePayload,
+        ForumTag as ForumTagPayload,
+        DefaultReaction as DefaultReactionPayload,
     )
     from .types.invite import Invite as InvitePayload
     from .types.role import Role as RolePayload
@@ -130,6 +133,44 @@ def _transform_roles(entry: AuditLogEntry, data: List[Snowflake]) -> List[Union[
     return [entry.guild.get_role(int(role_id)) or Object(role_id, type=Role) for role_id in data]
 
 
+def _transform_applied_forum_tags(entry: AuditLogEntry, data: List[Snowflake]) -> List[Union[ForumTag, Object]]:
+    thread = entry.target
+    if isinstance(thread, Thread) and isinstance(thread.parent, ForumChannel):
+        return [thread.parent.get_tag(tag_id) or Object(id=tag_id, type=ForumTag) for tag_id in map(int, data)]
+    return [Object(id=tag_id, type=ForumTag) for tag_id in data]
+
+
+def _transform_overloaded_flags(entry: AuditLogEntry, data: int) -> Union[int, flags.ChannelFlags]:
+    # The `flags` key is definitely overloaded. Right now it's for channels and threads but
+    # I am aware of `member.flags` and `user.flags` existing. However, this does not impact audit logs
+    # at the moment but better safe than sorry.
+    channel_audit_log_types = (
+        enums.AuditLogAction.channel_create,
+        enums.AuditLogAction.channel_update,
+        enums.AuditLogAction.channel_delete,
+        enums.AuditLogAction.thread_create,
+        enums.AuditLogAction.thread_update,
+        enums.AuditLogAction.thread_delete,
+    )
+
+    if entry.action in channel_audit_log_types:
+        return flags.ChannelFlags._from_value(data)
+    return data
+
+
+def _transform_forum_tags(entry: AuditLogEntry, data: List[ForumTagPayload]) -> List[ForumTag]:
+    return [ForumTag.from_data(state=entry._state, data=d) for d in data]
+
+
+def _transform_default_reaction(entry: AuditLogEntry, data: DefaultReactionPayload) -> Optional[PartialEmoji]:
+    if data is None:
+        return None
+
+    emoji_name = data.get('emoji_name') or ''
+    emoji_id = utils._get_as_snowflake(data, 'emoji_id') or None  # Coerce 0 -> None
+    return PartialEmoji.with_state(state=entry._state, name=emoji_name, id=emoji_id)
+
+
 def _transform_overwrites(
     entry: AuditLogEntry, data: List[PermissionOverwritePayload]
 ) -> List[Tuple[Object, PermissionOverwrite]]:
@@ -198,10 +239,6 @@ def _transform_automod_trigger_metadata(
         except Exception:
             pass
 
-    # If cannot get trigger type from the rule and data is empty, then cannot determine trigger type
-    if not data:
-        return None
-
     # Try to infer trigger type from available keys in data
     if 'presets' in data:
         return AutoModTrigger(
@@ -210,9 +247,16 @@ def _transform_automod_trigger_metadata(
             allow_list=data.get('allow_list'),
         )
     elif 'keyword_filter' in data:
-        return AutoModTrigger(type=enums.AutoModRuleTriggerType.keyword, keyword_filter=data['keyword_filter'])  # type: ignore
+        return AutoModTrigger(
+            type=enums.AutoModRuleTriggerType.keyword,
+            keyword_filter=data['keyword_filter'],  # type: ignore
+            allow_list=data.get('allow_list'),
+            regex_patterns=data.get('regex_patterns'),
+        )
     elif 'mention_total_limit' in data:
         return AutoModTrigger(type=enums.AutoModRuleTriggerType.mention_spam, mention_limit=data['mention_total_limit'])  # type: ignore
+    else:
+        return AutoModTrigger(type=enums.AutoModRuleTriggerType.spam)
 
 
 def _transform_automod_actions(entry: AuditLogEntry, data: List[AutoModerationAction]) -> List[AutoModRuleAction]:
@@ -239,9 +283,15 @@ def _flag_transformer(cls: Type[F]) -> Callable[[AuditLogEntry, Union[int, str]]
     return _transform
 
 
-def _transform_type(entry: AuditLogEntry, data: int) -> Union[enums.ChannelType, enums.StickerType]:
+def _transform_type(
+    entry: AuditLogEntry, data: Union[int, str]
+) -> Union[enums.ChannelType, enums.StickerType, enums.WebhookType, str]:
     if entry.action.name.startswith('sticker_'):
         return enums.try_enum(enums.StickerType, data)
+    elif entry.action.name.startswith('integration_'):
+        return data  # type: ignore  # integration type is str
+    elif entry.action.name.startswith('webhook_'):
+        return enums.try_enum(enums.WebhookType, data)
     else:
         return enums.try_enum(enums.ChannelType, data)
 
@@ -271,50 +321,55 @@ Transformer = Callable[["AuditLogEntry", Any], Any]
 
 class AuditLogChanges:
     # fmt: off
-    TRANSFORMERS: ClassVar[Dict[str, Tuple[Optional[str], Optional[Transformer]]]] = {
-        'verification_level':            (None, _enum_transformer(enums.VerificationLevel)),
-        'explicit_content_filter':       (None, _enum_transformer(enums.ContentFilter)),
-        'allow':                         (None, _flag_transformer(Permissions)),
-        'deny':                          (None, _flag_transformer(Permissions)),
-        'permissions':                   (None, _flag_transformer(Permissions)),
-        'id':                            (None, _transform_snowflake),
-        'color':                         ('colour', _transform_color),
-        'owner_id':                      ('owner', _transform_member_id),
-        'inviter_id':                    ('inviter', _transform_member_id),
-        'channel_id':                    ('channel', _transform_channel),
-        'afk_channel_id':                ('afk_channel', _transform_channel),
-        'system_channel_id':             ('system_channel', _transform_channel),
-        'system_channel_flags':          (None, _flag_transformer(flags.SystemChannelFlags)),
-        'widget_channel_id':             ('widget_channel', _transform_channel),
-        'rules_channel_id':              ('rules_channel', _transform_channel),
-        'public_updates_channel_id':     ('public_updates_channel', _transform_channel),
-        'permission_overwrites':         ('overwrites', _transform_overwrites),
-        'splash_hash':                   ('splash', _guild_hash_transformer('splashes')),
-        'banner_hash':                   ('banner', _guild_hash_transformer('banners')),
-        'discovery_splash_hash':         ('discovery_splash', _guild_hash_transformer('discovery-splashes')),
-        'icon_hash':                     ('icon', _transform_icon),
-        'avatar_hash':                   ('avatar', _transform_avatar),
-        'rate_limit_per_user':           ('slowmode_delay', None),
-        'guild_id':                      ('guild', _transform_guild_id),
-        'tags':                          ('emoji', None),
-        'default_message_notifications': ('default_notifications', _enum_transformer(enums.NotificationLevel)),
-        'video_quality_mode':            (None, _enum_transformer(enums.VideoQualityMode)),
-        'privacy_level':                 (None, _enum_transformer(enums.PrivacyLevel)),
-        'format_type':                   (None, _enum_transformer(enums.StickerFormatType)),
-        'type':                          (None, _transform_type),
-        'communication_disabled_until':  ('timed_out_until', _transform_timestamp),
-        'expire_behavior':               (None, _enum_transformer(enums.ExpireBehaviour)),
-        'mfa_level':                     (None, _enum_transformer(enums.MFALevel)),
-        'status':                        (None, _enum_transformer(enums.EventStatus)),
-        'entity_type':                   (None, _enum_transformer(enums.EntityType)),
-        'preferred_locale':              (None, _enum_transformer(enums.Locale)),
-        'image_hash':                    ('cover_image', _transform_cover_image),
-        'trigger_type':                  (None, _enum_transformer(enums.AutoModRuleTriggerType)),
-        'event_type':                    (None, _enum_transformer(enums.AutoModRuleEventType)),
-        'trigger_metadata':              ('trigger', _transform_automod_trigger_metadata),
-        'actions':                       (None, _transform_automod_actions),
-        'exempt_channels':               (None, _transform_channels_or_threads),
-        'exempt_roles':                  (None, _transform_roles),
+    TRANSFORMERS: ClassVar[Mapping[str, Tuple[Optional[str], Optional[Transformer]]]] = {
+        'verification_level':                    (None, _enum_transformer(enums.VerificationLevel)),
+        'explicit_content_filter':               (None, _enum_transformer(enums.ContentFilter)),
+        'allow':                                 (None, _flag_transformer(Permissions)),
+        'deny':                                  (None, _flag_transformer(Permissions)),
+        'permissions':                           (None, _flag_transformer(Permissions)),
+        'id':                                    (None, _transform_snowflake),
+        'color':                                 ('colour', _transform_color),
+        'owner_id':                              ('owner', _transform_member_id),
+        'inviter_id':                            ('inviter', _transform_member_id),
+        'channel_id':                            ('channel', _transform_channel),
+        'afk_channel_id':                        ('afk_channel', _transform_channel),
+        'system_channel_id':                     ('system_channel', _transform_channel),
+        'system_channel_flags':                  (None, _flag_transformer(flags.SystemChannelFlags)),
+        'widget_channel_id':                     ('widget_channel', _transform_channel),
+        'rules_channel_id':                      ('rules_channel', _transform_channel),
+        'public_updates_channel_id':             ('public_updates_channel', _transform_channel),
+        'permission_overwrites':                 ('overwrites', _transform_overwrites),
+        'splash_hash':                           ('splash', _guild_hash_transformer('splashes')),
+        'banner_hash':                           ('banner', _guild_hash_transformer('banners')),
+        'discovery_splash_hash':                 ('discovery_splash', _guild_hash_transformer('discovery-splashes')),
+        'icon_hash':                             ('icon', _transform_icon),
+        'avatar_hash':                           ('avatar', _transform_avatar),
+        'rate_limit_per_user':                   ('slowmode_delay', None),
+        'default_thread_rate_limit_per_user':    ('default_thread_slowmode_delay', None),
+        'guild_id':                              ('guild', _transform_guild_id),
+        'tags':                                  ('emoji', None),
+        'default_message_notifications':         ('default_notifications', _enum_transformer(enums.NotificationLevel)),
+        'video_quality_mode':                    (None, _enum_transformer(enums.VideoQualityMode)),
+        'privacy_level':                         (None, _enum_transformer(enums.PrivacyLevel)),
+        'format_type':                           (None, _enum_transformer(enums.StickerFormatType)),
+        'type':                                  (None, _transform_type),
+        'communication_disabled_until':          ('timed_out_until', _transform_timestamp),
+        'expire_behavior':                       (None, _enum_transformer(enums.ExpireBehaviour)),
+        'mfa_level':                             (None, _enum_transformer(enums.MFALevel)),
+        'status':                                (None, _enum_transformer(enums.EventStatus)),
+        'entity_type':                           (None, _enum_transformer(enums.EntityType)),
+        'preferred_locale':                      (None, _enum_transformer(enums.Locale)),
+        'image_hash':                            ('cover_image', _transform_cover_image),
+        'trigger_type':                          (None, _enum_transformer(enums.AutoModRuleTriggerType)),
+        'event_type':                            (None, _enum_transformer(enums.AutoModRuleEventType)),
+        'trigger_metadata':                      ('trigger', _transform_automod_trigger_metadata),
+        'actions':                               (None, _transform_automod_actions),
+        'exempt_channels':                       (None, _transform_channels_or_threads),
+        'exempt_roles':                          (None, _transform_roles),
+        'applied_tags':                          (None, _transform_applied_forum_tags),
+        'available_tags':                        (None, _transform_forum_tags),
+        'flags':                                 (None, _transform_overloaded_flags),
+        'default_reaction_emoji':                (None, _transform_default_reaction),
     }
     # fmt: on
 
@@ -495,11 +550,17 @@ class AuditLogEntry(Hashable):
     -----------
     action: :class:`AuditLogAction`
         The action that was done.
-    user: :class:`abc.User`
+    user: Optional[:class:`abc.User`]
         The user who initiated this action. Usually a :class:`Member`\, unless gone
         then it's a :class:`User`.
+    user_id: Optional[:class:`int`]
+        The user ID who initiated this action.
+
+        .. versionadded:: 2.2
     id: :class:`int`
         The entry ID.
+    guild: :class:`Guild`
+        The guild that this entry belongs to.
     target: Any
         The target that got changed. The exact type of this depends on
         the action being done.
@@ -515,19 +576,19 @@ class AuditLogEntry(Hashable):
     def __init__(
         self,
         *,
-        users: Dict[int, User],
-        integrations: Dict[int, PartialIntegration],
-        app_commands: Dict[int, AppCommand],
-        automod_rules: Dict[int, AutoModRule],
+        users: Mapping[int, User],
+        integrations: Mapping[int, PartialIntegration],
+        app_commands: Mapping[int, AppCommand],
+        automod_rules: Mapping[int, AutoModRule],
         data: AuditLogEntryPayload,
         guild: Guild,
     ):
         self._state: ConnectionState = guild._state
         self.guild: Guild = guild
-        self._users: Dict[int, User] = users
-        self._integrations: Dict[int, PartialIntegration] = integrations
-        self._app_commands: Dict[int, AppCommand] = app_commands
-        self._automod_rules: Dict[int, AutoModRule] = automod_rules
+        self._users: Mapping[int, User] = users
+        self._integrations: Mapping[int, PartialIntegration] = integrations
+        self._app_commands: Mapping[int, AppCommand] = app_commands
+        self._automod_rules: Mapping[int, AutoModRule] = automod_rules
         self._from_data(data)
 
     def _from_data(self, data: AuditLogEntryPayload) -> None:
@@ -578,7 +639,11 @@ class AuditLogEntry(Hashable):
                     channel=self.guild.get_channel_or_thread(channel_id) or Object(id=channel_id),
                     message_id=int(extra['message_id']),
                 )
-            elif self.action is enums.AuditLogAction.automod_block_message:
+            elif (
+                self.action is enums.AuditLogAction.automod_block_message
+                or self.action is enums.AuditLogAction.automod_flag_message
+                or self.action is enums.AuditLogAction.automod_timeout_member
+            ):
                 channel_id = int(extra['channel_id'])
                 self.extra = _AuditLogProxyAutoModAction(
                     automod_rule_name=extra['auto_moderation_rule_name'],
@@ -616,8 +681,8 @@ class AuditLogEntry(Hashable):
         # into meaningful data when requested
         self._changes = data.get('changes', [])
 
-        user_id = utils._get_as_snowflake(data, 'user_id')
-        self.user: Optional[Union[User, Member]] = self._get_member(user_id)
+        self.user_id: Optional[int] = utils._get_as_snowflake(data, 'user_id')
+        self.user: Optional[Union[User, Member]] = self._get_member(self.user_id)
         self._target_id = utils._get_as_snowflake(data, 'target_id')
 
     def _get_member(self, user_id: Optional[int]) -> Union[Member, User, None]:
@@ -658,11 +723,12 @@ class AuditLogEntry(Hashable):
         if self.action.target_type is None:
             return None
 
+        if self._target_id is None:
+            return None
+
         try:
             converter = getattr(self, '_convert_target_' + self.action.target_type)
         except AttributeError:
-            if self._target_id is None:
-                return None
             return Object(id=self._target_id)
         else:
             return converter(self._target_id)
@@ -695,8 +761,8 @@ class AuditLogEntry(Hashable):
     def _convert_target_channel(self, target_id: int) -> Union[abc.GuildChannel, Object]:
         return self.guild.get_channel(target_id) or Object(id=target_id)
 
-    def _convert_target_user(self, target_id: int) -> Union[Member, User, None]:
-        return self._get_member(target_id)
+    def _convert_target_user(self, target_id: int) -> Union[Member, User, Object]:
+        return self._get_member(target_id) or Object(id=target_id, type=Member)
 
     def _convert_target_role(self, target_id: int) -> Union[Role, Object]:
         return self.guild.get_role(target_id) or Object(id=target_id, type=Role)
@@ -725,8 +791,8 @@ class AuditLogEntry(Hashable):
     def _convert_target_emoji(self, target_id: int) -> Union[Emoji, Object]:
         return self._state.get_emoji(target_id) or Object(id=target_id, type=Emoji)
 
-    def _convert_target_message(self, target_id: int) -> Union[Member, User, None]:
-        return self._get_member(target_id)
+    def _convert_target_message(self, target_id: int) -> Union[Member, User, Object]:
+        return self._get_member(target_id) or Object(id=target_id, type=Member)
 
     def _convert_target_stage_instance(self, target_id: int) -> Union[StageInstance, Object]:
         return self.guild.get_stage_instance(target_id) or Object(id=target_id, type=StageInstance)
@@ -757,6 +823,9 @@ class AuditLogEntry(Hashable):
         target = self._get_integration_by_app_id(target_id) or self._get_app_command(target_id)
         if not target:
             try:
+                # circular import
+                from .app_commands import AppCommand
+
                 # get application id from extras
                 # if it matches target id, type should be integration
                 target_app = self.extra
