@@ -26,7 +26,7 @@ from __future__ import annotations
 import inspect
 import discord
 from discord import app_commands
-from discord.utils import maybe_coroutine
+from discord.utils import maybe_coroutine, _to_kebab_case
 
 from typing import (
     Any,
@@ -39,6 +39,7 @@ from typing import (
     List,
     Optional,
     TYPE_CHECKING,
+    Sequence,
     Tuple,
     TypeVar,
     Union,
@@ -49,6 +50,7 @@ from ._types import _BaseCommand, BotT
 if TYPE_CHECKING:
     from typing_extensions import Self
     from discord.abc import Snowflake
+    from discord._types import ClientT
 
     from .bot import BotBase
     from .context import Context
@@ -145,6 +147,12 @@ class CogMeta(type):
         than :class:`str`. Defaults to ``True``.
 
         .. versionadded:: 2.0
+    group_extras: :class:`dict`
+        A dictionary that can be used to store extraneous data.
+        This is only applicable for :class:`GroupCog` instances.
+        The library will not touch any values or keys within this dictionary.
+
+        .. versionadded:: 2.1
     """
 
     __cog_name__: str
@@ -153,6 +161,7 @@ class CogMeta(type):
     __cog_group_description__: Union[str, app_commands.locale_str]
     __cog_group_nsfw__: bool
     __cog_group_auto_locale_strings__: bool
+    __cog_group_extras__: Dict[Any, Any]
     __cog_settings__: Dict[str, Any]
     __cog_commands__: List[Command[Any, ..., Any]]
     __cog_app_commands__: List[Union[app_commands.Group, app_commands.Command[Any, ..., Any]]]
@@ -174,7 +183,7 @@ class CogMeta(type):
             try:
                 group_name = kwargs.pop('group_name')
             except KeyError:
-                group_name = app_commands.commands._to_kebab_case(name)
+                group_name = _to_kebab_case(name)
         else:
             group_name = kwargs.pop('group_name', cog_name)
 
@@ -183,6 +192,7 @@ class CogMeta(type):
         attrs['__cog_group_name__'] = group_name
         attrs['__cog_group_nsfw__'] = kwargs.pop('group_nsfw', False)
         attrs['__cog_group_auto_locale_strings__'] = kwargs.pop('group_auto_locale_strings', True)
+        attrs['__cog_group_extras__'] = kwargs.pop('group_extras', {})
 
         description = kwargs.pop('description', None)
         if description is None:
@@ -306,6 +316,7 @@ class Cog(metaclass=CogMeta):
                 guild_ids=getattr(cls, '__discord_app_commands_default_guilds__', None),
                 guild_only=getattr(cls, '__discord_app_commands_guild_only__', False),
                 default_permissions=getattr(cls, '__discord_app_commands_default_permissions__', None),
+                extras=cls.__cog_group_extras__,
             )
         else:
             group = None
@@ -335,7 +346,7 @@ class Cog(metaclass=CogMeta):
                     command.app_command = app_command  # type: ignore
 
                     if self.__cog_app_commands_group__:
-                        children.append(app_command)
+                        children.append(app_command)  # type: ignore # Somehow it thinks it can be None here
 
         if Cog._get_overridden_method(self.cog_app_command_error) is not None:
             error_handler = self.cog_app_command_error
@@ -481,7 +492,7 @@ class Cog(metaclass=CogMeta):
         """
 
         if name is not MISSING and not isinstance(name, str):
-            raise TypeError(f'Cog.listener expected str but received {name.__class__.__name__!r} instead.')
+            raise TypeError(f'Cog.listener expected str but received {name.__class__.__name__} instead.')
 
         def decorator(func: FuncT) -> FuncT:
             actual = func
@@ -510,6 +521,13 @@ class Cog(metaclass=CogMeta):
         """
         return not hasattr(self.cog_command_error.__func__, '__cog_special_method__')
 
+    def has_app_command_error_handler(self) -> bool:
+        """:class:`bool`: Checks whether the cog has an app error handler.
+
+        .. versionadded:: 2.1
+        """
+        return not hasattr(self.cog_app_command_error.__func__, '__cog_special_method__')
+
     @_cog_special_method
     async def cog_load(self) -> None:
         """|maybecoro|
@@ -568,9 +586,23 @@ class Cog(metaclass=CogMeta):
         """
         return True
 
+    @_cog_special_method
+    def interaction_check(self, interaction: discord.Interaction[ClientT], /) -> bool:
+        """A special method that registers as a :func:`discord.app_commands.check`
+        for every app command and subcommand in this cog.
+
+        This function **can** be a coroutine and must take a sole parameter,
+        ``interaction``, to represent the :class:`~discord.Interaction`.
+
+        .. versionadded:: 2.0
+        """
+        return True
+
     @_cog_special_method
     async def cog_command_error(self, ctx: Context[BotT], error: Exception) -> None:
-        """A special method that is called whenever an error
+        """|coro|
+
+        A special method that is called whenever an error
         is dispatched inside this cog.
 
         This is similar to :func:`.on_command_error` except only applying
@@ -589,7 +621,9 @@ class Cog(metaclass=CogMeta):
 
     @_cog_special_method
     async def cog_app_command_error(self, interaction: discord.Interaction, error: app_commands.AppCommandError) -> None:
-        """A special method that is called whenever an error within
+        """|coro|
+
+        A special method that is called whenever an error within
         an application command is dispatched inside this cog.
 
         This is similar to :func:`discord.app_commands.CommandTree.on_error` except
@@ -608,7 +642,9 @@ class Cog(metaclass=CogMeta):
 
     @_cog_special_method
     async def cog_before_invoke(self, ctx: Context[BotT]) -> None:
-        """A special method that acts as a cog local pre-invoke hook.
+        """|coro|
+
+        A special method that acts as a cog local pre-invoke hook.
 
         This is similar to :meth:`.Command.before_invoke`.
 
@@ -623,7 +659,9 @@ class Cog(metaclass=CogMeta):
 
     @_cog_special_method
     async def cog_after_invoke(self, ctx: Context[BotT]) -> None:
-        """A special method that acts as a cog local post-invoke hook.
+        """|coro|
+
+        A special method that acts as a cog local post-invoke hook.
 
         This is similar to :meth:`.Command.after_invoke`.
 
@@ -636,7 +674,7 @@ class Cog(metaclass=CogMeta):
         """
         pass
 
-    async def _inject(self, bot: BotBase, override: bool, guild: Optional[Snowflake], guilds: List[Snowflake]) -> Self:
+    async def _inject(self, bot: BotBase, override: bool, guild: Optional[Snowflake], guilds: Sequence[Snowflake]) -> Self:
         cls = self.__class__
 
         # we'll call this first so that errors can propagate without
@@ -727,6 +765,9 @@ class GroupCog(Cog):
     and :func:`~discord.app_commands.default_permissions` will apply to the group if used on top of the
     cog.
 
+    Hybrid commands will also be added to the Group, giving the ability to categorize slash commands into
+    groups, while keeping the prefix-style command as a root-level command.
+
     For example:
 
     .. code-block:: python3
