@@ -272,7 +272,7 @@ def _set_api_version(value: int):
     global INTERNAL_API_VERSION
 
     if not isinstance(value, int):
-        raise TypeError(f'expected int not {value.__class__!r}')
+        raise TypeError(f'expected int not {value.__class__.__name__}')
 
     if value not in (9, 10):
         raise ValueError(f'expected either 9 or 10 not {value}')
@@ -1032,12 +1032,12 @@ class HTTPClient:
         self,
         user_id: Snowflake,
         guild_id: Snowflake,
-        delete_message_days: int = 1,
+        delete_message_seconds: int = 86400,  # one day
         reason: Optional[str] = None,
     ) -> Response[None]:
         r = Route('PUT', '/guilds/{guild_id}/bans/{user_id}', guild_id=guild_id, user_id=user_id)
         params = {
-            'delete_message_days': delete_message_days,
+            'delete_message_seconds': delete_message_seconds,
         }
 
         return self.request(r, params=params, reason=reason)
@@ -1143,7 +1143,13 @@ class HTTPClient:
             'invitable',
             'default_auto_archive_duration',
             'flags',
+            'default_thread_rate_limit_per_user',
+            'default_reaction_emoji',
+            'available_tags',
+            'applied_tags',
+            'default_forum_layout',
         )
+
         payload = {k: v for k, v in options.items() if k in valid_keys}
         return self.request(r, reason=reason, json=payload)
 
@@ -1182,6 +1188,8 @@ class HTTPClient:
             'rtc_region',
             'video_quality_mode',
             'default_auto_archive_duration',
+            'default_thread_rate_limit_per_user',
+            'available_tags',
         )
         payload.update({k: v for k, v in options.items() if k in valid_keys and v is not None})
 
@@ -1709,7 +1717,7 @@ class HTTPClient:
         params: Dict[str, Any] = {'limit': limit}
         if before:
             params['before'] = before
-        if after:
+        if after is not None:
             params['after'] = after
         if user_id:
             params['user_id'] = user_id
