@@ -25,7 +25,7 @@ DEALINGS IN THE SOFTWARE.
 from __future__ import annotations
 
 from datetime import datetime
-from typing import TYPE_CHECKING, AsyncIterator, Dict, Optional, Union
+from typing import TYPE_CHECKING, AsyncIterator, Dict, Optional, Union, overload, Literal
 
 from .asset import Asset
 from .enums import EventStatus, EntityType, PrivacyLevel, try_enum
@@ -98,6 +98,10 @@ class ScheduledEvent(Hashable):
         The number of users subscribed to the scheduled event.
     creator: Optional[:class:`User`]
         The user that created the scheduled event.
+    creator_id: Optional[:class:`int`]
+        The ID of the user that created the scheduled event.
+
+        .. versionadded:: 2.2
     location: Optional[:class:`str`]
         The location of the scheduled event.
     """
@@ -119,6 +123,7 @@ class ScheduledEvent(Hashable):
         'user_count',
         'creator',
         'channel_id',
+        'creator_id',
         'location',
     )
 
@@ -139,10 +144,14 @@ class ScheduledEvent(Hashable):
         self.status: EventStatus = try_enum(EventStatus, data['status'])
         self._cover_image: Optional[str] = data.get('image', None)
         self.user_count: int = data.get('user_count', 0)
+        self.creator_id: Optional[int] = _get_as_snowflake(data, 'creator_id')
 
         creator = data.get('creator')
         self.creator: Optional[User] = self._state.store_user(creator) if creator else None
 
+        if self.creator_id is not None and self.creator is None:
+            self.creator = self._state.get_user(self.creator_id)
+
         self.end_time: Optional[datetime] = parse_time(data.get('scheduled_end_time'))
         self.channel_id: Optional[int] = _get_as_snowflake(data, 'channel_id')
 
@@ -152,13 +161,6 @@ class ScheduledEvent(Hashable):
     def _unroll_metadata(self, data: Optional[EntityMetadata]):
         self.location: Optional[str] = data.get('location') if data else None
 
-    @classmethod
-    def from_creation(cls, *, state: ConnectionState, data: GuildScheduledEventPayload) -> None:
-        creator_id = data.get('creator_id')
-        self = cls(state=state, data=data)
-        if creator_id:
-            self.creator = self._state.get_user(int(creator_id))
-
     def __repr__(self) -> str:
         return f'<GuildScheduledEvent id={self.id} name={self.name!r} guild_id={self.guild_id!r} creator={self.creator!r}>'
 
@@ -296,6 +298,87 @@ class ScheduledEvent(Hashable):
 
         return await self.__modify_status(EventStatus.cancelled, reason)
 
+    @overload
+    async def edit(
+        self,
+        *,
+        name: str = ...,
+        description: str = ...,
+        start_time: datetime = ...,
+        end_time: Optional[datetime] = ...,
+        privacy_level: PrivacyLevel = ...,
+        status: EventStatus = ...,
+        image: bytes = ...,
+        reason: Optional[str] = ...,
+    ) -> ScheduledEvent:
+        ...
+
+    @overload
+    async def edit(
+        self,
+        *,
+        name: str = ...,
+        description: str = ...,
+        channel: Snowflake,
+        start_time: datetime = ...,
+        end_time: Optional[datetime] = ...,
+        privacy_level: PrivacyLevel = ...,
+        entity_type: Literal[EntityType.voice, EntityType.stage_instance],
+        status: EventStatus = ...,
+        image: bytes = ...,
+        reason: Optional[str] = ...,
+    ) -> ScheduledEvent:
+        ...
+
+    @overload
+    async def edit(
+        self,
+        *,
+        name: str = ...,
+        description: str = ...,
+        start_time: datetime = ...,
+        end_time: datetime = ...,
+        privacy_level: PrivacyLevel = ...,
+        entity_type: Literal[EntityType.external],
+        status: EventStatus = ...,
+        image: bytes = ...,
+        location: str,
+        reason: Optional[str] = ...,
+    ) -> ScheduledEvent:
+        ...
+
+    @overload
+    async def edit(
+        self,
+        *,
+        name: str = ...,
+        description: str = ...,
+        channel: Union[VoiceChannel, StageChannel],
+        start_time: datetime = ...,
+        end_time: Optional[datetime] = ...,
+        privacy_level: PrivacyLevel = ...,
+        status: EventStatus = ...,
+        image: bytes = ...,
+        reason: Optional[str] = ...,
+    ) -> ScheduledEvent:
+        ...
+
+    @overload
+    async def edit(
+        self,
+        *,
+        name: str = ...,
+        description: str = ...,
+        start_time: datetime = ...,
+        end_time: datetime = ...,
+        privacy_level: PrivacyLevel = ...,
+        status: EventStatus = ...,
+        image: bytes = ...,
+        location: str,
+        reason: Optional[str] = ...,
+    ) -> ScheduledEvent:
+        ...
+
     async def edit(
         self,
         *,
@@ -412,6 +495,15 @@ class ScheduledEvent(Hashable):
             payload['image'] = image_as_str
 
         entity_type = entity_type or getattr(channel, '_scheduled_event_entity_type', MISSING)
+        if entity_type is MISSING:
+            if channel and isinstance(channel, Object):
+                if channel.type is VoiceChannel:
+                    entity_type = EntityType.voice
+                elif channel.type is StageChannel:
+                    entity_type = EntityType.stage_instance
+            elif location not in (MISSING, None):
+                entity_type = EntityType.external
+
         if entity_type is None:
             raise TypeError(
                 f'invalid GuildChannel type passed, must be VoiceChannel or StageChannel not {channel.__class__.__name__}'
@@ -424,12 +516,14 @@ class ScheduledEvent(Hashable):
             payload['entity_type'] = entity_type.value
 
         _entity_type = entity_type or self.entity_type
+        _entity_type_changed = _entity_type is not self.entity_type
 
         if _entity_type in (EntityType.stage_instance, EntityType.voice):
             if channel is MISSING or channel is None:
-                raise TypeError('channel must be set when entity_type is voice or stage_instance')
-
-            payload['channel_id'] = channel.id
+                if _entity_type_changed:
+                    raise TypeError('channel must be set when entity_type is voice or stage_instance')
+            else:
+                payload['channel_id'] = channel.id
 
             if location not in (MISSING, None):
                 raise TypeError('location cannot be set when entity_type is voice or stage_instance')
@@ -440,11 +534,12 @@ class ScheduledEvent(Hashable):
             payload['channel_id'] = None
 
             if location is MISSING or location is None:
-                raise TypeError('location must be set when entity_type is external')
+                if _entity_type_changed:
+                    raise TypeError('location must be set when entity_type is external')
+            else:
+                metadata['location'] = location
 
-            metadata['location'] = location
-
-            if end_time is MISSING or end_time is None:
+            if not self.end_time and (end_time is MISSING or end_time is None):
                 raise TypeError('end_time must be set when entity_type is external')
 
         if end_time is not MISSING:
@@ -560,25 +655,27 @@ class ScheduledEvent(Hashable):
                 predicate = lambda u: u['user']['id'] > after.id
 
         while True:
-            retrieve = min(100 if limit is None else limit, 100)
+            retrieve = 100 if limit is None else min(limit, 100)
             if retrieve < 1:
                 return
 
             data, state, limit = await strategy(retrieve, state, limit)
 
-            if len(data) < 100:
-                limit = 0
-
             if reverse:
                 data = reversed(data)
             if predicate:
                 data = filter(predicate, data)
 
             users = (self._state.store_user(raw_user['user']) for raw_user in data)
+            count = 0
 
-            for user in users:
+            for count, user in enumerate(users, 1):
                 yield user
 
+            if count < 100:
+                # There's no data left after this
+                break
+
     def _add_user(self, user: User) -> None:
         self._users[user.id] = user
 
